@@ -18,16 +18,20 @@
 package net.floodlightcontroller.routing;
 
 import java.io.IOException;
+import java.util.EnumSet;
 import java.util.ArrayList;
 import java.util.Comparator;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
 
 import net.floodlightcontroller.core.FloodlightContext;
 import net.floodlightcontroller.core.IFloodlightProviderService;
 import net.floodlightcontroller.core.IOFMessageListener;
 import net.floodlightcontroller.core.IOFSwitch;
+import net.floodlightcontroller.core.annotations.LogMessageCategory;
+import net.floodlightcontroller.core.annotations.LogMessageDoc;
+import net.floodlightcontroller.core.annotations.LogMessageDocs;
 import net.floodlightcontroller.core.util.AppCookie;
 import net.floodlightcontroller.counter.ICounterStoreService;
 import net.floodlightcontroller.devicemanager.IDevice;
@@ -35,11 +39,13 @@ import net.floodlightcontroller.devicemanager.IDeviceListener;
 import net.floodlightcontroller.devicemanager.IDeviceService;
 import net.floodlightcontroller.devicemanager.SwitchPort;
 import net.floodlightcontroller.packet.Ethernet;
+import net.floodlightcontroller.packet.IPacket;
 import net.floodlightcontroller.routing.IRoutingService;
 import net.floodlightcontroller.routing.IRoutingDecision;
 import net.floodlightcontroller.routing.Route;
 import net.floodlightcontroller.topology.ITopologyService;
 import net.floodlightcontroller.topology.NodePortTuple;
+import net.floodlightcontroller.util.OFMessageDamper;
 import net.floodlightcontroller.util.TimedCache;
 
 import org.openflow.protocol.OFFlowMod;
@@ -47,33 +53,44 @@ import org.openflow.protocol.OFMatch;
 import org.openflow.protocol.OFMessage;
 import org.openflow.protocol.OFPacketIn;
 import org.openflow.protocol.OFPacketOut;
-import org.openflow.protocol.OFPort;
 import org.openflow.protocol.OFType;
 import org.openflow.protocol.action.OFAction;
 import org.openflow.protocol.action.OFActionOutput;
-import org.openflow.util.U16;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-public abstract class ForwardingBase implements
-    IOFMessageListener,
-    IDeviceListener {
+/**
+ * Abstract base class for implementing a forwarding module.  Forwarding is
+ * responsible for programming flows to a switch in response to a policy
+ * decision.
+ */
+@LogMessageCategory("Flow Programming")
+public abstract class ForwardingBase 
+    implements IOFMessageListener, IDeviceListener {
+    
     protected static Logger log =
             LoggerFactory.getLogger(ForwardingBase.class);
 
-    public static final short FLOWMOD_DEFAULT_HARD_TIMEOUT = 5; // in seconds
+    protected static int OFMESSAGE_DAMPER_CAPACITY = 50000; // TODO: find sweet spot
+    protected static int OFMESSAGE_DAMPER_TIMEOUT = 250; // ms 
 
+    public static short FLOWMOD_DEFAULT_IDLE_TIMEOUT = 5; // in seconds
+    public static short FLOWMOD_DEFAULT_HARD_TIMEOUT = 0; // infinite
+    
     protected IFloodlightProviderService floodlightProvider;
     protected IDeviceService deviceManager;
     protected IRoutingService routingEngine;
     protected ITopologyService topology;
     protected ICounterStoreService counterStore;
     
+    protected OFMessageDamper messageDamper;
+    
     // for broadcast loop suppression
     protected boolean broadcastCacheFeature = true;
-    public final int prime = 2633;  // for hash calculation
+    public final int prime1 = 2633;  // for hash calculation
+    public final static int prime2 = 4357;  // for hash calculation
     public TimedCache<Long> broadcastCache =
-    		new TimedCache<Long>(100, 5*1000);  // 5 seconds interval;
+        new TimedCache<Long>(100, 5*1000);  // 5 seconds interval;
 
     // flow-mod - for use in the cookie
     public static final int FORWARDING_APP_ID = 2; // TODO: This must be managed
@@ -92,11 +109,21 @@ public abstract class ForwardingBase implements
                     return d1ClusterId.compareTo(d2ClusterId);
                 }
             };
+            
+    /**
+     * init data structures
+     * 
+     */
+    protected void init() {
+        messageDamper = new OFMessageDamper(OFMESSAGE_DAMPER_CAPACITY, 
+                                            EnumSet.of(OFType.FLOW_MOD),
+                                            OFMESSAGE_DAMPER_TIMEOUT);
+    }
 
     /**
      * Adds a listener for devicemanager and registers for PacketIns.
      */
-    public void startUp() {
+    protected void startUp() {
         deviceManager.addListener(this);
         floodlightProvider.addOFMessageListener(OFType.PACKET_IN, this);
     }
@@ -140,10 +167,9 @@ public abstract class ForwardingBase implements
                                                    (OFPacketIn) msg,
                                                    decision,
                                                    cntx);
+            default:
+                break;
         }
-        log.error("received an unexpected message {} from switch {}",
-                  msg,
-                  sw);
         return Command.CONTINUE;
     }
 
@@ -153,7 +179,6 @@ public abstract class ForwardingBase implements
      * @param match OpenFlow fields to match on
      * @param srcSwPort Source switch port for the first hop
      * @param dstSwPort Destination switch port for final hop
-     * @param bufferId BufferId of the original PacketIn
      * @param cookie The cookie to set in each flow_mod
      * @param cntx The floodlight context
      * @param reqeustFlowRemovedNotifn if set to true then the switch would
@@ -164,9 +189,20 @@ public abstract class ForwardingBase implements
      *        OFFlowMod.OFPFC_MODIFY etc.
      * @return srcSwitchIincluded True if the source switch is included in this route
      */
+    @LogMessageDocs({
+        @LogMessageDoc(level="WARN",
+            message="Unable to push route, switch at DPID {dpid} not available",
+            explanation="A switch along the calculated path for the " +
+                        "flow has disconnected.",
+            recommendation=LogMessageDoc.CHECK_SWITCH),
+        @LogMessageDoc(level="ERROR",
+            message="Failure writing flow mod",
+            explanation="An I/O error occurred while writing a " +
+                        "flow modification to a switch",
+            recommendation=LogMessageDoc.CHECK_SWITCH)            
+    })
     public boolean pushRoute(Route route, OFMatch match, 
                              Integer wildcard_hints,
-                             int bufferId,
                              OFPacketIn pi,
                              long pinSwitch,
                              long cookie, 
@@ -180,10 +216,12 @@ public abstract class ForwardingBase implements
                 (OFFlowMod) floodlightProvider.getOFMessageFactory()
                                               .getMessage(OFType.FLOW_MOD);
         OFActionOutput action = new OFActionOutput();
+        action.setMaxLength((short)0xffff);
         List<OFAction> actions = new ArrayList<OFAction>();
         actions.add(action);
 
-        fm.setIdleTimeout((short)5)
+        fm.setIdleTimeout(FLOWMOD_DEFAULT_IDLE_TIMEOUT)
+            .setHardTimeout(FLOWMOD_DEFAULT_HARD_TIMEOUT)
             .setBufferId(OFPacketOut.BUFFER_ID_NONE)
             .setCookie(cookie)
             .setCommand(flowModCommand)
@@ -210,9 +248,6 @@ public abstract class ForwardingBase implements
 
             // set buffer id if it is the source switch
             if (1 == indx) {
-                //fm.setMatch(match);
-                fm.setBufferId(bufferId);
-                //fm.setMatch(wildcard(match, sw, wildcard_hints));
                 // Set the flag to request flow-mod removal notifications only for the
                 // source switch. The removal message is used to maintain the flow
                 // cache. Don't set the flag for ARP messages - TODO generalize check
@@ -239,13 +274,15 @@ public abstract class ForwardingBase implements
                                           fm.getMatch().getInputPort(),
                                           outPort });
                 }
-                sw.write(fm, cntx);
+                messageDamper.write(sw, fm, cntx);
                 if (doFlush) {
                     sw.flush();
                 }
 
                 // Push the packet out the source switch
                 if (sw.getId() == pinSwitch) {
+                    // TODO: Instead of doing a packetOut here we could also 
+                    // send a flowMod with bufferId set.... 
                     pushPacket(sw, match, pi, outPort, cntx);
                     srcSwitchIncluded = true;
                 }
@@ -270,6 +307,88 @@ public abstract class ForwardingBase implements
         }
         return match.clone();
     }
+    
+    /**
+     * Pushes a packet-out to a switch. If bufferId != BUFFER_ID_NONE we 
+     * assume that the packetOut switch is the same as the packetIn switch
+     * and we will use the bufferId 
+     * Caller needs to make sure that inPort and outPort differs
+     * @param packet    packet data to send
+     * @param sw        switch from which packet-out is sent
+     * @param bufferId  bufferId
+     * @param inPort    input port
+     * @param outPort   output port
+     * @param cntx      context of the packet
+     * @param flush     force to flush the packet.
+     */
+    @LogMessageDocs({
+        @LogMessageDoc(level="ERROR",
+            message="BufferId is not and packet data is null. " +
+                    "Cannot send packetOut. " +
+                    "srcSwitch={dpid} inPort={port} outPort={port}",
+            explanation="The switch send a malformed packet-in." +
+                        "The packet will be dropped",
+            recommendation=LogMessageDoc.REPORT_SWITCH_BUG),
+        @LogMessageDoc(level="ERROR",
+            message="Failure writing packet out",
+            explanation="An I/O error occurred while writing a " +
+                    "packet out to a switch",
+            recommendation=LogMessageDoc.CHECK_SWITCH)            
+    })
+    public void pushPacket(IPacket packet, 
+                           IOFSwitch sw,
+                           int bufferId,
+                           short inPort,
+                           short outPort, 
+                           FloodlightContext cntx,
+                           boolean flush) {
+        
+        
+        if (log.isTraceEnabled()) {
+            log.trace("PacketOut srcSwitch={} inPort={} outPort={}", 
+                      new Object[] {sw, inPort, outPort});
+        }
+
+        OFPacketOut po =
+                (OFPacketOut) floodlightProvider.getOFMessageFactory()
+                                                .getMessage(OFType.PACKET_OUT);
+
+        // set actions
+        List<OFAction> actions = new ArrayList<OFAction>();
+        actions.add(new OFActionOutput(outPort, (short) 0xffff));
+
+        po.setActions(actions)
+          .setActionsLength((short) OFActionOutput.MINIMUM_LENGTH);
+        short poLength =
+                (short) (po.getActionsLength() + OFPacketOut.MINIMUM_LENGTH);
+
+        // set buffer_id, in_port
+        po.setBufferId(bufferId);
+        po.setInPort(inPort);
+
+        // set data - only if buffer_id == -1
+        if (po.getBufferId() == OFPacketOut.BUFFER_ID_NONE) {
+            if (packet == null) {
+                log.error("BufferId is not set and packet data is null. " +
+                          "Cannot send packetOut. " +
+                        "srcSwitch={} inPort={} outPort={}",
+                        new Object[] {sw, inPort, outPort});
+                return;
+            }
+            byte[] packetData = packet.serialize();
+            poLength += packetData.length;
+            po.setPacketData(packetData);
+        }
+
+        po.setLength(poLength);
+
+        try {
+            counterStore.updatePktOutFMCounterStore(sw, po);
+            messageDamper.write(sw, po, cntx, flush);
+        } catch (IOException e) {
+            log.error("Failure writing packet out", e);
+        }
+    }
 
     /**
      * Pushes a packet-out to a switch.  The assumption here is that
@@ -282,13 +401,17 @@ public abstract class ForwardingBase implements
      * @param outport   output port
      * @param cntx      context of the packet
      */
-    public void pushPacket(IOFSwitch sw, OFMatch match, OFPacketIn pi, 
+    protected void pushPacket(IOFSwitch sw, OFMatch match, OFPacketIn pi, 
                            short outport, FloodlightContext cntx) {
-        
+
         if (pi == null) {
             return;
+        } else if (pi.getInPort() == outport){
+            log.warn("Packet out not sent as the outport matches inport. {}",
+                     pi);
+            return;
         }
-        
+
         // The assumption here is (sw) is the switch that generated the 
         // packet-in. If the input port is the same as output port, then
         // the packet-out should be ignored.
@@ -301,7 +424,7 @@ public abstract class ForwardingBase implements
                 return;
             }
         }
-        
+
         if (log.isTraceEnabled()) {
             log.trace("PacketOut srcSwitch={} match={} pi={}", 
                       new Object[] {sw, match, pi});
@@ -313,18 +436,27 @@ public abstract class ForwardingBase implements
 
         // set actions
         List<OFAction> actions = new ArrayList<OFAction>();
-        actions.add(new OFActionOutput(outport, (short) 0));
+        actions.add(new OFActionOutput(outport, (short) 0xffff));
 
         po.setActions(actions)
           .setActionsLength((short) OFActionOutput.MINIMUM_LENGTH);
         short poLength =
                 (short) (po.getActionsLength() + OFPacketOut.MINIMUM_LENGTH);
 
-        // set buffer_id, in_port
-        po.setBufferId(pi.getBufferId());
+        // If the switch doens't support buffering set the buffer id to be none
+        // otherwise it'll be the the buffer id of the PacketIn
+        if (sw.getBuffers() == 0) {
+            // We set the PI buffer id here so we don't have to check again below
+            pi.setBufferId(OFPacketOut.BUFFER_ID_NONE);
+            po.setBufferId(OFPacketOut.BUFFER_ID_NONE);
+        } else {
+            po.setBufferId(pi.getBufferId());
+        }
+
         po.setInPort(pi.getInPort());
 
-        // set data - only if buffer_id == -1
+        // If the buffer id is none or the switch doesn's support buffering
+        // we send the data with the packet out
         if (pi.getBufferId() == OFPacketOut.BUFFER_ID_NONE) {
             byte[] packetData = pi.getPacketData();
             poLength += packetData.length;
@@ -335,26 +467,26 @@ public abstract class ForwardingBase implements
 
         try {
             counterStore.updatePktOutFMCounterStore(sw, po);
-            sw.write(po, cntx);
+            messageDamper.write(sw, po, cntx);
         } catch (IOException e) {
             log.error("Failure writing packet out", e);
         }
     }
 
+    
     /**
      * Write packetout message to sw with output actions to one or more
      * output ports with inPort/outPorts passed in.
-     * Note that the packet in could be from a different switch.
-     * @param pi
+     * @param packetData
      * @param sw
      * @param inPort
      * @param ports
      * @param cntx
      */
-    public void PacketOutMultiPort(OFPacketIn pi,
+    public void packetOutMultiPort(byte[] packetData,
                                    IOFSwitch sw,
                                    short inPort,
-                                   HashSet<Integer> outPorts,
+                                   Set<Integer> outPorts,
                                    FloodlightContext cntx) {
         //setting actions
         List<OFAction> actions = new ArrayList<OFAction>();
@@ -381,7 +513,6 @@ public abstract class ForwardingBase implements
         // data (note buffer_id is always BUFFER_ID_NONE) and length
         short poLength = (short)(po.getActionsLength() + 
                 OFPacketOut.MINIMUM_LENGTH);
-        byte[] packetData = pi.getPacketData();
         poLength += packetData.length;
         po.setPacketData(packetData);
         po.setLength(poLength);
@@ -390,18 +521,44 @@ public abstract class ForwardingBase implements
             counterStore.updatePktOutFMCounterStore(sw, po);
             if (log.isTraceEnabled()) {
                 log.trace("write broadcast packet on switch-id={} " + 
-                        "interaces={} packet-in={} packet-out={}",
-                        new Object[] {sw.getId(), outPorts, pi, po});
+                        "interfaces={} packet-out={}",
+                        new Object[] {sw.getId(), outPorts, po});
             }
-            sw.write(po, cntx);
+            messageDamper.write(sw, po, cntx);
 
         } catch (IOException e) {
             log.error("Failure writing packet out", e);
         }
     }
+    
+    /** 
+     * @see packetOutMultiPort
+     * Accepts a PacketIn instead of raw packet data. Note that the inPort
+     * and switch can be different than the packet in switch/port
+     */
+    public void packetOutMultiPort(OFPacketIn pi,
+                                   IOFSwitch sw,
+                                   short inPort,
+                                   Set<Integer> outPorts,
+                                   FloodlightContext cntx) {
+        packetOutMultiPort(pi.getPacketData(), sw, inPort, outPorts, cntx);
+    }
+    
+    /** 
+     * @see packetOutMultiPort
+     * Accepts an IPacket instead of raw packet data. Note that the inPort
+     * and switch can be different than the packet in switch/port
+     */
+    public void packetOutMultiPort(IPacket packet,
+                                   IOFSwitch sw,
+                                   short inPort,
+                                   Set<Integer> outPorts,
+                                   FloodlightContext cntx) {
+        packetOutMultiPort(packet.serialize(), sw, inPort, outPorts, cntx);
+    }
 
     protected boolean isInBroadcastCache(IOFSwitch sw, OFPacketIn pi,
-    		FloodlightContext cntx) {
+                        FloodlightContext cntx) {
         // Get the cluster id of the switch.
         // Get the hash of the Ethernet packet.
         if (sw == null) return true;  
@@ -411,11 +568,11 @@ public abstract class ForwardingBase implements
 
         Ethernet eth = 
             IFloodlightProviderService.bcStore.get(cntx,
-            		IFloodlightProviderService.CONTEXT_PI_PAYLOAD);
+                IFloodlightProviderService.CONTEXT_PI_PAYLOAD);
         
         Long broadcastHash;
-        broadcastHash = topology.getL2DomainId(sw.getId())
-        		* prime + eth.hashCode();
+        broadcastHash = topology.getL2DomainId(sw.getId()) * prime1 +
+                        pi.getInPort() * prime2 + eth.hashCode();
         if (broadcastCache.update(broadcastHash)) {
             sw.updateBroadcastCache(broadcastHash, pi.getInPort());
             return true;
@@ -429,18 +586,24 @@ public abstract class ForwardingBase implements
         
         // If the feature is disabled, always return false;
         if (!broadcastCacheFeature) return false;
-        
+
         // Get the hash of the Ethernet packet.
         Ethernet eth =
                 IFloodlightProviderService.bcStore.get(cntx, IFloodlightProviderService.CONTEXT_PI_PAYLOAD);
 
-        // some FORWARD_OR_FLOOD packets are unicast with unknown destination mac
-        // if (eth.isBroadcast() || eth.isMulticast())
-            return sw.updateBroadcastCache(new Long(eth.hashCode()), pi.getInPort());
+        long hash =  pi.getInPort() * prime2 + eth.hashCode();
 
-        // return false;
+        // some FORWARD_OR_FLOOD packets are unicast with unknown destination mac
+        return sw.updateBroadcastCache(hash, pi.getInPort());
     }
 
+    @LogMessageDocs({
+        @LogMessageDoc(level="ERROR",
+            message="Failure writing deny flow mod",
+            explanation="An I/O error occurred while writing a " +
+                    "deny flow mod to a switch",
+            recommendation=LogMessageDoc.CHECK_SWITCH)            
+    })
     public static boolean
             blockHost(IFloodlightProviderService floodlightProvider,
                       SwitchPort sw_tup, long host_mac,
@@ -470,7 +633,8 @@ public abstract class ForwardingBase implements
                      & ~OFMatch.OFPFW_IN_PORT);
         fm.setCookie(cookie)
           .setHardTimeout((short) hardTimeout)
-          .setIdleTimeout((short) 5)
+          .setIdleTimeout(FLOWMOD_DEFAULT_IDLE_TIMEOUT)
+          .setHardTimeout(FLOWMOD_DEFAULT_HARD_TIMEOUT)
           .setBufferId(OFPacketOut.BUFFER_ID_NONE)
           .setMatch(match)
           .setActions(actions)
@@ -479,6 +643,7 @@ public abstract class ForwardingBase implements
         try {
             log.debug("write drop flow-mod sw={} match={} flow-mod={}",
                       new Object[] { sw, match, fm });
+            // TODO: can't use the message damper sine this method is static
             sw.write(fm, null);
         } catch (IOException e) {
             log.error("Failure writing deny flow mod", e);
@@ -488,40 +653,6 @@ public abstract class ForwardingBase implements
 
     }
 
-    /**
-     * @param floodlightProvider the floodlightProvider to set
-     */
-    public void setFloodlightProvider(IFloodlightProviderService floodlightProvider) {
-        this.floodlightProvider = floodlightProvider;
-    }
-
-    /**
-     * @param routingEngine the routingEngine to set
-     */
-    public void setRoutingEngine(IRoutingService routingEngine) {
-        this.routingEngine = routingEngine;
-    }
-
-    /**
-     * @param deviceManager
-     *            the deviceManager to set
-     */
-    public void setDeviceManager(IDeviceService deviceManager) {
-        this.deviceManager = deviceManager;
-    }
-
-    /**
-     * @param topology
-     *            the topology to set
-     */
-    public void setTopology(ITopologyService topology) {
-        this.topology = topology;
-    }
-
-    public void setCounterStore(ICounterStoreService counterStore) {
-        this.counterStore = counterStore;
-    }
-
     @Override
     public void deviceAdded(IDevice device) {
         // NOOP
@@ -534,29 +665,6 @@ public abstract class ForwardingBase implements
 
     @Override
     public void deviceMoved(IDevice device) {
-        // Build flow mod to delete based on destination mac == device mac
-        OFMatch match = new OFMatch();
-        match.setDataLayerDestination(Ethernet.toByteArray(device.getMACAddress()));
-        match.setWildcards(OFMatch.OFPFW_ALL ^ OFMatch.OFPFW_DL_DST);
-        long cookie =
-                AppCookie.makeCookie(FORWARDING_APP_ID, 0);
-        OFMessage fm = ((OFFlowMod) floodlightProvider.getOFMessageFactory()
-            .getMessage(OFType.FLOW_MOD))
-            .setCommand(OFFlowMod.OFPFC_DELETE)
-            .setOutPort((short) OFPort.OFPP_NONE.getValue())
-            .setMatch(match)
-            .setCookie(cookie)
-            .setLength(U16.t(OFFlowMod.MINIMUM_LENGTH));
-
-        // Flush to all switches
-        for (IOFSwitch outSw : floodlightProvider.getSwitches().values()) {
-            try {
-                outSw.write(fm, null);
-            } catch (IOException e) {
-                log.error("Failure sending flow mod delete for moved device",
-                          e);
-            }
-        }
     }
 
     @Override
