@@ -15,12 +15,19 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import net.floodlightcontroller.util.ClusterDFS;
+import net.floodlightcontroller.core.annotations.LogMessageCategory;
+import net.floodlightcontroller.core.annotations.LogMessageDoc;
 import net.floodlightcontroller.routing.BroadcastTree;
 import net.floodlightcontroller.routing.Link;
 import net.floodlightcontroller.routing.Route;
 import net.floodlightcontroller.routing.RouteId;
 import net.floodlightcontroller.util.LRUHashMap;
 
+/**
+ * A representation of a network topology.  Used internally by 
+ * {@link TopologyManager}
+ */
+@LogMessageCategory("Network Topology")
 public class TopologyInstance {
 
     public static final short LT_SH_LINK = 1;
@@ -144,14 +151,14 @@ public class TopologyInstance {
     }
 
     public void printTopology() {
-        log.debug("-----------------------------------------------");
-        log.debug("Links: {}",this.switchPortLinks);
-        log.debug("broadcastDomainPorts: {}", broadcastDomainPorts);
-        log.debug("tunnelPorts: {}", tunnelPorts);
-        log.debug("clusters: {}", clusters);
-        log.debug("destinationRootedTrees: {}", destinationRootedTrees);
-        log.debug("clusterBroadcastNodePorts: {}", clusterBroadcastNodePorts);
-        log.debug("-----------------------------------------------");
+        log.trace("-----------------------------------------------");
+        log.trace("Links: {}",this.switchPortLinks);
+        log.trace("broadcastDomainPorts: {}", broadcastDomainPorts);
+        log.trace("tunnelPorts: {}", tunnelPorts);
+        log.trace("clusters: {}", clusters);
+        log.trace("destinationRootedTrees: {}", destinationRootedTrees);
+        log.trace("clusterBroadcastNodePorts: {}", clusterBroadcastNodePorts);
+        log.trace("-----------------------------------------------");
     }
 
     protected void addLinksToOpenflowDomains() {
@@ -188,6 +195,10 @@ public class TopologyInstance {
      *
      * http://en.wikipedia.org/wiki/Tarjan%27s_strongly_connected_components_algorithm
      */
+    @LogMessageDoc(level="ERROR",
+            message="No DFS object for switch {} found.",
+            explanation="The internal state of the topology module is corrupt",
+            recommendation=LogMessageDoc.REPORT_CONTROLLER_BUG)
     public void identifyOpenflowDomains() {
         Map<Long, ClusterDFS> dfsList = new HashMap<Long, ClusterDFS>();
 
@@ -741,7 +752,7 @@ public class TopologyInstance {
         return switches;
     }
 
-    public Set<Short> getPorts(long sw) {
+    public Set<Short> getPortsWithLinks(long sw) {
         return switchPorts.get(sw);
     }
 
